/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package com.funambol.syncml.protocol;

/**
 * The interface <i>SyncFilter</i> is used by the SyncManager to apply a
 * filter defined for a SyncSource to the outgoing request.
 * <p>
 * An implementation of SyncSource can contain an implementation of SyncFilter,
 * that must be able to provide a SyncML 1.2 filter when the method toString()
 * is called.
 */
public interface SyncFilter {
    
    //------------------------------------------------------------ Attributes
    
    //--------------------------------------------------------------- Methods

    /**
     * Format the Filter in SyncML syntax, according to the parameter
     * contained in the SyncFilter implementation.
     *
     * @return the SyncML filter to add to the Alert command.
     */
    public String toSyncML();

    /**
     * Format the Filter in SyncML syntax. This is done according to the
     * parameter contained in the SyncFilter implementation, but with a size
     * limit forced by the sync engine. This limit is usually maxMsgSize
     * or MaxObjSize.
     *
     * @return the SyncML filter to add to the Alert command.
     */
    public String toSyncML(int maxSize);

}

