/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package com.funambol.syncml.client;

import jmunit.framework.cldc10.TestCase;

import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.SyncManager;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.syncml.spds.SourceConfig;

import com.funambol.storage.NamedObjectStore;
import com.funambol.util.Log;

public class SyncTest extends TestCase {

    private static final String STORE_NAME = "TESTCONFIG";
    private static final String SOURCE_NAME = "source.briefcase";
    private static final String URL  = "http://localhost:8080/funambol/ds";
    
    private SourceConfig sc;
    private SyncConfig conf;

    public SyncTest() {
        super(6, "SyncTest");
        
        Log.setLogLevel(Log.DEBUG);

        NamedObjectStore os = new NamedObjectStore();
        sc = new SourceConfig();
        conf = new SyncConfig();

        try {
            os.create(STORE_NAME);
            // Set defaults
            conf.syncUrl = URL;
            os.store(SyncConfig.NAME, conf);

            // Set defaults
            sc.setType("text/plain");
            sc.setEncoding(SyncSource.ENCODING_NONE);
            os.store(SOURCE_NAME, sc);
        }
        catch (Exception e) {
            e.printStackTrace();
            Log.error("Can't init config.");
        }
    }
    
    public void setUp() {
    }

    public void tearDown() {
    }
    
    public void test(int testNumber) throws Throwable {

        NamedObjectStore os = new NamedObjectStore();
        conf = new SyncConfig();
        sc = new SourceConfig();

        try {
            os.open(STORE_NAME);
            os.retrieve(SyncConfig.NAME, conf);
            os.retrieve(SOURCE_NAME, sc);
        }
        catch(Exception e) {
            Log.debug("Error retrieving conf: "+e.toString());
            // Set defaults
            conf.syncUrl = URL;
            sc.setType("text/plain");
            sc.setEncoding(SyncSource.ENCODING_NONE);
        }

        SyncManager sm = new SyncManager(conf);
        TestSyncSource testsrc = new TestSyncSource(sc);

        switch(testNumber) {
            case 0: 
                testSlow(sm, testsrc);
                break;
            case 1: 
                testFast(sm, testsrc);
                break;
            case 2: 
                testOneWayFromClient(sm, testsrc);        
                break;
            case 3: 
                testOneWayFromServer(sm, testsrc);        
                break;
            case 4: 
                testRefreshFromClient(sm, testsrc);
                break;
            case 5: 
                testRefreshFromServer(sm, testsrc);
                break;
            default:                    
                break;
        }

        // Save Anchors
        os.store(SOURCE_NAME, sc);

        os.close();
        os = null;
    }

    public void testSlow(SyncManager sm, SyncSource src) throws Exception {
        Log.info("------------ BEGIN TEST SLOW --------------");
        sm.sync(src, SyncML.ALERT_CODE_SLOW);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST SLOW ---------------");
    }

    public void testFast(SyncManager sm, SyncSource src) throws Exception {
        Log.info("------------ BEGIN TEST FAST --------------");
        sm.sync(src);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST FAST --------------");
    }

    public void testOneWayFromClient(SyncManager sm, SyncSource src)
    throws Exception {
        Log.info("------------ BEGIN TEST ONE WAY FROM CLIENT --------------");
        sm.sync(src, SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST ONE WAY FROM CLIENT --------------");
    }

    public void testOneWayFromServer(SyncManager sm, SyncSource src)
    throws Exception {
        Log.info("------------ BEGIN TEST ONE WAY FROM SERVER --------------");
        sm.sync(src, SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST ONE WAY FROM SERVER --------------");
    }

    public void testRefreshFromClient(SyncManager sm, SyncSource src)
    throws Exception {
        Log.info("------------ BEGIN TEST REFRESH FROM CLIENT --------------");
        sm.sync(src, SyncML.ALERT_CODE_REFRESH_FROM_CLIENT);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST REFRESH FROM CLIENT --------------");
    }
    
    public void testRefreshFromServer(SyncManager sm, SyncSource src)
    throws Exception {
        Log.info("------------ BEGIN TEST REFRESH FROM SERVER --------------");
        sm.sync(src, SyncML.ALERT_CODE_REFRESH_FROM_SERVER);
        Log.info("Client items: "+src.getClientItemsNumber());
        Log.info("Server items: "+src.getServerItemsNumber());
        Log.info("------------- END TEST REFRESH FROM SERVER --------------");
    }

}

